@@ -7,11 +7,13 @@
 #include "nsXULAppAPI.h"
 #include "jsapi.h"
 #include "jsfriendapi.h"
-#include "js/Array.h"  // JS::NewArrayObject
+#include "js/Array.h"             // JS::NewArrayObject
+#include "js/CallAndConstruct.h"  // JS_CallFunctionValue
 #include "js/CharacterEncoding.h"
 #include "js/CompilationAndEvaluation.h"  // JS::Evaluate
 #include "js/ContextOptions.h"
 #include "js/Printf.h"
+#include "js/PropertyAndElement.h"  // JS_DefineElement, JS_DefineFunctions, JS_DefineProperty
 #include "js/PropertySpec.h"
 #include "js/SourceText.h"  // JS::SourceText
 #include "mozilla/ChaosMode.h"
@@ -35,6 +37,7 @@
 #include "nsDirectoryServiceUtils.h"
 #include "nsCOMPtr.h"
 #include "nsJSPrincipals.h"
+#include "nsJSUtils.h"
 #include "xpcpublic.h"
 #include "xpcprivate.h"
 #include "BackstagePass.h"
@@ -44,8 +47,8 @@
 
 #include "nsIXULRuntime.h"
 #include "nsIAppStartup.h"
-#include "GeckoProfiler.h"
 #include "Components.h"
+#include "ProfilerControl.h"
 
 #ifdef ANDROID
 #  include <android/log.h>
@@ -57,6 +60,7 @@
 #  include "mozilla/ScopeExit.h"
 #  include "mozilla/widget/AudioSession.h"
 #  include "mozilla/WinDllServices.h"
+#  include "mozilla/WindowsBCryptInitialization.h"
 #  include <windows.h>
 #  if defined(MOZ_SANDBOX)
 #    include "XREShellData.h"
@@ -116,15 +120,11 @@ class XPCShellDirProvider : public nsIDirectoryServiceProvider2 {
   // The app executable
   void SetAppFile(nsIFile* appFile);
   void ClearAppFile() { mAppFile = nullptr; }
-  // An additional custom plugin dir if specified
-  void SetPluginDir(nsIFile* pluginDir);
-  void ClearPluginDir() { mPluginDir = nullptr; }
 
  private:
   nsCOMPtr<nsIFile> mGREDir;
   nsCOMPtr<nsIFile> mGREBinDir;
   nsCOMPtr<nsIFile> mAppDir;
-  nsCOMPtr<nsIFile> mPluginDir;
   nsCOMPtr<nsIFile> mAppFile;
 };
 
@@ -164,18 +164,9 @@ static bool GetLocationProperty(JSContext* cx, unsigned argc, Value* vp) {
 #else
   JS::AutoFilename filename;
   if (JS::DescribeScriptedCaller(cx, &filename) && filename.get()) {
-#  if defined(XP_WIN)
-    // convert from the system codepage to UTF-16
-    int bufferSize =
-        MultiByteToWideChar(CP_ACP, 0, filename.get(), -1, nullptr, 0);
-    nsAutoString filenameString;
-    filenameString.SetLength(bufferSize);
-    MultiByteToWideChar(CP_ACP, 0, filename.get(), -1,
-                        (LPWSTR)filenameString.BeginWriting(),
-                        filenameString.Length());
-    // remove the null terminator
-    filenameString.SetLength(bufferSize - 1);
+    NS_ConvertUTF8toUTF16 filenameString(filename.get());
 
+#  if defined(XP_WIN)
     // replace forward slashes with backslashes,
     // since nsLocalFileWin chokes on them
     char16_t* start = filenameString.BeginWriting();
@@ -187,8 +178,6 @@ static bool GetLocationProperty(JSContext* cx, unsigned argc, Value* vp) {
       }
       start++;
     }
-#  elif defined(XP_UNIX)
-    NS_ConvertUTF8toUTF16 filenameString(filename.get());
 #  endif
 
     nsCOMPtr<nsIFile> location;
@@ -375,28 +364,14 @@ static bool Load(JSContext* cx, unsigned argc, Value* vp) {
     if (!str) {
       return false;
     }
-    JS::UniqueChars filename = JS_EncodeStringToLatin1(cx, str);
+    JS::UniqueChars filename = JS_EncodeStringToUTF8(cx, str);
     if (!filename) {
       return false;
     }
-    FILE* file = fopen(filename.get(), "r");
-    if (!file) {
-      filename = JS_EncodeStringToUTF8(cx, str);
-      if (!filename) {
-        return false;
-      }
-      JS_ReportErrorUTF8(cx, "cannot open file '%s' for reading",
-                         filename.get());
-      return false;
-    }
     JS::CompileOptions options(cx);
-    options.setFileAndLine(filename.get(), 1)
-        .setIsRunOnce(true)
-        .setSkipFilenameValidation(true);
-    JS::Rooted<JSScript*> script(cx);
-    JS::Rooted<JSObject*> global(cx, JS::CurrentGlobalOrNull(cx));
-    script = JS::CompileUtf8File(cx, options, file);
-    fclose(file);
+    options.setIsRunOnce(true).setSkipFilenameValidation(true);
+    JS::Rooted<JSScript*> script(
+        cx, JS::CompileUtf8Path(cx, options, filename.get()));
     if (!script) {
       return false;
     }
@@ -636,6 +611,37 @@ static bool RegisterAppManifest(JSContext* cx, unsigned argc, Value* vp) {
   return true;
 }
 
+#ifdef ANDROID
+static bool ChangeTestShellDir(JSContext* cx, unsigned argc, Value* vp) {
+  // This method should only be used by testing/xpcshell/head.js to change to
+  // the correct directory on Android Remote XPCShell tests.
+  //
+  // TODO: Bug 1801725 - Find a more ergonomic way to do this than exposing
+  // identical methods in XPCShellEnvironment and XPCShellImpl to chdir on
+  // android for Remote XPCShell tests on Android.
+  CallArgs args = CallArgsFromVp(argc, vp);
+
+  if (args.length() != 1) {
+    JS_ReportErrorASCII(cx, "changeTestShellDir() takes one argument");
+    return false;
+  }
+
+  nsAutoJSCString path;
+  if (!path.init(cx, args[0])) {
+    JS_ReportErrorASCII(
+        cx, "changeTestShellDir(): could not convert argument 1 to string");
+    return false;
+  }
+
+  if (chdir(path.get())) {
+    JS_ReportErrorASCII(cx, "changeTestShellDir(): could not change directory");
+    return false;
+  }
+
+  return true;
+}
+#endif
+
 #ifdef ENABLE_TESTS
 static bool RegisterXPCTestComponents(JSContext* cx, unsigned argc, Value* vp) {
   JS::CallArgs args = JS::CallArgsFromVp(argc, vp);
@@ -643,7 +649,7 @@ static bool RegisterXPCTestComponents(JSContext* cx, unsigned argc, Value* vp) {
     JS_ReportErrorASCII(cx, "Wrong number of arguments");
     return false;
   }
-  nsresult rv = XRE_AddStaticComponent(&kXPCTestModule);
+  nsresult rv = xpcTestRegisterComponents();
   if (NS_FAILED(rv)) {
     XPCThrower::Throw(rv, cx);
     return false;
@@ -671,6 +677,9 @@ static const JSFunctionSpec glob_functions[] = {
     JS_FN("setInterruptCallback", SetInterruptCallback, 1,0),
     JS_FN("simulateNoScriptActivity", SimulateNoScriptActivity, 1,0),
     JS_FN("registerAppManifest", RegisterAppManifest, 1, 0),
+#ifdef ANDROID
+    JS_FN("changeTestShellDir", ChangeTestShellDir, 1,0),
+#endif
 #ifdef ENABLE_TESTS
     JS_FN("registerXPCTestComponents", RegisterXPCTestComponents, 0, 0),
 #endif
@@ -759,9 +768,7 @@ static bool ProcessFile(AutoJSAPI& jsapi, const char* filename, FILE* file,
      * It's not interactive - just execute it.
      *
      * Support the UNIX #! shell hack; gobble the first line if it starts
-     * with '#'.  TODO - this isn't quite compatible with sharp variables,
-     * as a legal js program (using sharp variables) might start with '#'.
-     * But that would require multi-character lookahead.
+     * with '#'.
      */
     int ch = fgetc(file);
     if (ch == '#') {
@@ -773,10 +780,15 @@ static bool ProcessFile(AutoJSAPI& jsapi, const char* filename, FILE* file,
     }
     ungetc(ch, file);
 
+    JS::UniqueChars filenameUtf8 = JS::EncodeNarrowToUtf8(jsapi.cx(), filename);
+    if (!filenameUtf8) {
+      return false;
+    }
+
     JS::RootedScript script(cx);
     JS::RootedValue unused(cx);
     JS::CompileOptions options(cx);
-    options.setFileAndLine(filename, 1)
+    options.setFileAndLine(filenameUtf8.get(), 1)
         .setIsRunOnce(true)
         .setNoScriptRval(true)
         .setSkipFilenameValidation(true);
@@ -988,18 +1000,6 @@ static bool ProcessArgs(AutoJSAPI& jsapi, char** argv, int argc,
         compileOnly = true;
         isInteractive = false;
         break;
-      case 'p': {
-        // plugins path
-        char* pluginPath = argv[++i];
-        nsCOMPtr<nsIFile> pluginsDir;
-        if (NS_FAILED(
-                XRE_GetFileFromPath(pluginPath, getter_AddRefs(pluginsDir)))) {
-          fprintf(gErrFile, "Couldn't use given plugins dir.\n");
-          return printUsageAndSetExitCode();
-        }
-        aDirProvider->SetPluginDir(pluginsDir);
-        break;
-      }
       default:
         return printUsageAndSetExitCode();
     }
@@ -1076,10 +1076,10 @@ int XRE_XPCShellMain(int argc, char** argv, char** envp,
   // with the IOInterposer will be properly tracked.
   mozilla::IOInterposerInit ioInterposerGuard;
 
-#ifdef MOZ_GECKO_PROFILER
+  XRE_InitCommandLine(argc, argv);
+
   char aLocal;
   profiler_init(&aLocal);
-#endif
 
 #ifdef MOZ_ASAN_REPORTER
   PR_SetEnv("MOZ_DISABLE_ASAN_REPORTER=1");
@@ -1309,14 +1309,14 @@ int XRE_XPCShellMain(int argc, char** argv, char** envp,
 
     // Initialize e10s check on the main thread, if not already done
     BrowserTabsRemoteAutostart();
-#ifdef XP_WIN
+#if defined(XP_WIN)
     // Plugin may require audio session if installed plugin can initialize
     // asynchronized.
     AutoAudioSession audioSession;
 
     // Ensure that DLL Services are running
     RefPtr<DllServices> dllSvc(DllServices::Get());
-    dllSvc->StartUntrustedModulesProcessor();
+    dllSvc->StartUntrustedModulesProcessor(true);
     auto dllServicesDisable =
         MakeScopeExit([&dllSvc]() { dllSvc->DisableFull(); });
 
@@ -1330,8 +1330,13 @@ int XRE_XPCShellMain(int argc, char** argv, char** envp,
           "Failed to initialize broker services, sandboxed "
           "processes will fail to start.");
     }
-#  endif
-#endif
+#  endif  // defined(MOZ_SANDBOX)
+
+    {
+      DebugOnly<bool> result = WindowsBCryptInitialization();
+      MOZ_ASSERT(result);
+    }
+#endif  // defined(XP_WIN)
 
 #ifdef MOZ_CODE_COVERAGE
     CodeCoverageHandler::Init();
@@ -1378,7 +1383,7 @@ int XRE_XPCShellMain(int argc, char** argv, char** envp,
 
           result = FuzzXPCRuntimeStart(&jsapi, &argc, &argv,
                                        aShellData->fuzzerDriver);
-#  elif __AFL_COMPILER
+#  elif AFLFUZZ
           MOZ_CRASH("AFL is unsupported for XPC runtime fuzzing integration");
 #  endif
         } else {
@@ -1420,7 +1425,6 @@ int XRE_XPCShellMain(int argc, char** argv, char** envp,
 
     dirprovider.ClearGREDirs();
     dirprovider.ClearAppDir();
-    dirprovider.ClearPluginDir();
     dirprovider.ClearAppFile();
   }  // this scopes the nsCOMPtrs
 
@@ -1438,14 +1442,14 @@ int XRE_XPCShellMain(int argc, char** argv, char** envp,
     CrashReporter::UnsetExceptionHandler();
   }
 
-#ifdef MOZ_GECKO_PROFILER
   // This must precede NS_LogTerm(), otherwise xpcshell return non-zero
   // during some tests, which causes failures.
   profiler_shutdown();
-#endif
 
   NS_LogTerm();
 
+  XRE_DeinitCommandLine();
+
   return result;
 }
 
@@ -1466,10 +1470,6 @@ void XPCShellDirProvider::SetAppFile(nsIFile* appFile) { mAppFile = appFile; }
 
 void XPCShellDirProvider::SetAppDir(nsIFile* appDir) { mAppDir = appDir; }
 
-void XPCShellDirProvider::SetPluginDir(nsIFile* pluginDir) {
-  mPluginDir = pluginDir;
-}
-
 NS_IMETHODIMP_(MozExternalRefCountType)
 XPCShellDirProvider::AddRef() { return 2; }
 
@@ -1534,29 +1534,6 @@ XPCShellDirProvider::GetFiles(const char* prop, nsISimpleEnumerator** result) {
       return NS_NewArrayEnumerator(result, dirs, NS_GET_IID(nsIFile));
     }
     return NS_ERROR_FAILURE;
-  } else if (!strcmp(prop, NS_APP_PLUGINS_DIR_LIST)) {
-    nsCOMArray<nsIFile> dirs;
-    // Add the test plugin location passed in by the caller or through
-    // runxpcshelltests.
-    if (mPluginDir) {
-      dirs.AppendObject(mPluginDir);
-      // If there was no path specified, default to the one set up by automation
-    } else {
-      nsCOMPtr<nsIFile> file;
-      bool exists;
-      // We have to add this path, buildbot copies the test plugin directory
-      // to (app)/bin when unpacking test zips.
-      if (mGREDir) {
-        mGREDir->Clone(getter_AddRefs(file));
-        if (NS_SUCCEEDED(mGREDir->Clone(getter_AddRefs(file)))) {
-          file->AppendNative("plugins"_ns);
-          if (NS_SUCCEEDED(file->Exists(&exists)) && exists) {
-            dirs.AppendObject(file);
-          }
-        }
-      }
-    }
-    return NS_NewArrayEnumerator(result, dirs, NS_GET_IID(nsIFile));
   }
   return NS_ERROR_FAILURE;
 }
