@@ -19,18 +19,22 @@
 #include "js/Object.h"             // JS::GetClass
 #include "js/ProfilingStack.h"
 #include "GeckoProfiler.h"
+#include "mozJSModuleLoader.h"
 #include "nsJSEnvironment.h"
 #include "nsThreadUtils.h"
 #include "nsDOMJSUtils.h"
 
 #include "WrapperFactory.h"
 #include "AccessCheck.h"
+#include "JSServices.h"
 
 #include "mozilla/BasePrincipal.h"
 #include "mozilla/dom/BindingUtils.h"
 #include "mozilla/dom/DOMException.h"
 #include "mozilla/dom/Exceptions.h"
 #include "mozilla/dom/Promise.h"
+#include "mozilla/glean/bindings/Glean.h"
+#include "mozilla/glean/bindings/GleanPings.h"
 #include "mozilla/ScriptPreloader.h"
 
 #include "nsDOMMutationObserver.h"
@@ -45,7 +49,11 @@
 #include "nsJSUtils.h"
 #include "prsystem.h"
 
-#ifndef XP_WIN
+#include "xpcprivate.h"
+
+#ifdef XP_WIN
+#  include "mozilla/WinHeaderOnlyUtils.h"
+#else
 #  include <sys/mman.h>
 #endif
 
@@ -70,7 +78,7 @@ const char XPC_SCRIPT_ERROR_CONTRACTID[] = "@mozilla.org/scripterror;1";
 
 /***************************************************************************/
 
-nsXPConnect::nsXPConnect() : mShuttingDown(false) {
+nsXPConnect::nsXPConnect() {
 #ifdef MOZ_GECKO_PROFILER
   JS::SetProfilingThreadCallbacks(profiler_register_thread,
                                   profiler_unregister_thread);
@@ -88,7 +96,7 @@ void nsXPConnect::InitJSContext() {
   gSelf->mContext = xpccx;
   gSelf->mRuntime = xpccx->Runtime();
 
-  mozJSComponentLoader::InitStatics();
+  mozJSModuleLoader::InitStatics();
 
   // Initialize the script preloader cache.
   Unused << mozilla::ScriptPreloader::GetSingleton();
@@ -109,17 +117,20 @@ nsXPConnect::~nsXPConnect() {
   // XPConnect, to clean the stuff we forcibly disconnected. The forced
   // shutdown code defaults to leaking in a number of situations, so we can't
   // get by with only the second GC. :-(
-  mRuntime->GarbageCollect(JS::GCReason::XPCONNECT_SHUTDOWN);
+  //
+  // Bug 1650075: These should really pass GCOptions::Shutdown but doing that
+  // seems to cause crashes.
+  mRuntime->GarbageCollect(JS::GCOptions::Normal,
+                           JS::GCReason::XPCONNECT_SHUTDOWN);
 
-  mShuttingDown = true;
   XPCWrappedNativeScope::SystemIsBeingShutDown();
-  mRuntime->SystemIsBeingShutDown();
 
   // The above causes us to clean up a bunch of XPConnect data structures,
   // after which point we need to GC to clean everything up. We need to do
   // this before deleting the XPCJSContext, because doing so destroys the
   // maps that our finalize callback depends on.
-  mRuntime->GarbageCollect(JS::GCReason::XPCONNECT_SHUTDOWN);
+  mRuntime->GarbageCollect(JS::GCOptions::Normal,
+                           JS::GCReason::XPCONNECT_SHUTDOWN);
 
   NS_RELEASE(gSystemPrincipal);
   gScriptSecurityManager = nullptr;
@@ -165,7 +176,7 @@ void nsXPConnect::ReleaseXPConnectSingleton() {
     NS_RELEASE2(xpc, cnt);
   }
 
-  mozJSComponentLoader::Shutdown();
+  mozJSModuleLoader::ShutdownLoaders();
 }
 
 // static
@@ -470,23 +481,22 @@ JSObject* CreateGlobalObject(JSContext* cx, const JSClass* clasp,
 }
 
 void InitGlobalObjectOptions(JS::RealmOptions& aOptions,
-                             nsIPrincipal* aPrincipal) {
+                             bool aIsSystemPrincipal,
+                             bool aShouldResistFingerprinting) {
   bool shouldDiscardSystemSource = ShouldDiscardSystemSource();
 
-  bool isSystem = aPrincipal->IsSystemPrincipal();
-
-  if (isSystem) {
+  if (aIsSystemPrincipal) {
     // Make toSource functions [ChromeOnly]
     aOptions.creationOptions().setToSourceEnabled(true);
     // Make sure [SecureContext] APIs are visible:
     aOptions.creationOptions().setSecureContext(true);
     aOptions.behaviors().setClampAndJitterTime(false);
   }
+  aOptions.behaviors().setShouldResistFingerprinting(
+      aShouldResistFingerprinting);
 
   if (shouldDiscardSystemSource) {
-    bool discardSource = isSystem;
-
-    aOptions.behaviors().setDiscardSource(discardSource);
+    aOptions.behaviors().setDiscardSource(aIsSystemPrincipal);
   }
 }
 
@@ -505,6 +515,10 @@ bool InitGlobalObject(JSContext* aJSContext, JS::Handle<JSObject*> aGlobal,
         !XPCNativeWrapper::AttachNewConstructorObject(aJSContext, aGlobal)) {
       return UnexpectedFailure(false);
     }
+
+    if (!mozJSModuleLoader::Get()->DefineJSServices(aJSContext, aGlobal)) {
+      return UnexpectedFailure(false);
+    }
   }
 
   if (!(aFlags & xpc::DONT_FIRE_ONNEWGLOBALHOOK)) {
@@ -526,8 +540,13 @@ nsresult InitClassesWithNewWrappedGlobal(JSContext* aJSContext,
   // We pass null for the 'extra' pointer during global object creation, so
   // we need to have a principal.
   MOZ_ASSERT(aPrincipal);
+  // All uses (at time of writing) were System Principal, meaning
+  // aShouldResistFingerprinting can be hardcoded to false.
+  // If this changes, ShouldRFP needs to be updated accordingly.
+  MOZ_RELEASE_ASSERT(aPrincipal->IsSystemPrincipal());
 
-  InitGlobalObjectOptions(aOptions, aPrincipal);
+  InitGlobalObjectOptions(aOptions, /* aSystemPrincipal */ true,
+                          /* aShouldResistFingerprinting */ false);
 
   // Call into XPCWrappedNative to make a new global object, scope, and global
   // prototype.
@@ -552,12 +571,81 @@ nsresult InitClassesWithNewWrappedGlobal(JSContext* aJSContext,
     if (!JS_DefineProfilingFunctions(aJSContext, global)) {
       return UnexpectedFailure(NS_ERROR_OUT_OF_MEMORY);
     }
+    if (aPrincipal->IsSystemPrincipal()) {
+      if (!glean::Glean::DefineGlean(aJSContext, global) ||
+          !glean::GleanPings::DefineGleanPings(aJSContext, global)) {
+        return UnexpectedFailure(NS_ERROR_FAILURE);
+      }
+    }
   }
 
   aNewGlobal.set(global);
   return NS_OK;
 }
 
+nsCString GetFunctionName(JSContext* cx, HandleObject obj) {
+  RootedObject inner(cx, js::UncheckedUnwrap(obj));
+  JSAutoRealm ar(cx, inner);
+
+  RootedFunction fun(cx, JS_GetObjectFunction(inner));
+  if (!fun) {
+    // If the object isn't a function, it's likely that it has a single
+    // function property (for things like nsITimerCallback). In this case,
+    // return the name of that function property.
+
+    Rooted<IdVector> idArray(cx, IdVector(cx));
+    if (!JS_Enumerate(cx, inner, &idArray)) {
+      JS_ClearPendingException(cx);
+      return nsCString("error");
+    }
+
+    if (idArray.length() != 1) {
+      return nsCString("nonfunction");
+    }
+
+    RootedId id(cx, idArray[0]);
+    RootedValue v(cx);
+    if (!JS_GetPropertyById(cx, inner, id, &v)) {
+      JS_ClearPendingException(cx);
+      return nsCString("nonfunction");
+    }
+
+    if (!v.isObject()) {
+      return nsCString("nonfunction");
+    }
+
+    RootedObject vobj(cx, &v.toObject());
+    return GetFunctionName(cx, vobj);
+  }
+
+  RootedString funName(cx, JS_GetFunctionDisplayId(fun));
+  RootedScript script(cx, JS_GetFunctionScript(cx, fun));
+  const char* filename = script ? JS_GetScriptFilename(script) : "anonymous";
+  const char* filenameSuffix = strrchr(filename, '/');
+
+  if (filenameSuffix) {
+    filenameSuffix++;
+  } else {
+    filenameSuffix = filename;
+  }
+
+  nsCString displayName("anonymous");
+  if (funName) {
+    RootedValue funNameVal(cx, StringValue(funName));
+    if (!XPCConvert::JSData2Native(cx, &displayName, funNameVal,
+                                   {nsXPTType::T_UTF8STRING}, nullptr, 0,
+                                   nullptr)) {
+      JS_ClearPendingException(cx);
+      return nsCString("anonymous");
+    }
+  }
+
+  displayName.Append('[');
+  displayName.Append(filenameSuffix, strlen(filenameSuffix));
+  displayName.Append(']');
+  return displayName;
+}
+
 }  // namespace xpc
 
 static nsresult NativeInterface2JSObject(JSContext* aCx, HandleObject aScope,
@@ -581,10 +669,9 @@ static nsresult NativeInterface2JSObject(JSContext* aCx, HandleObject aScope,
   return NS_OK;
 }
 
-NS_IMETHODIMP
-nsXPConnect::WrapNative(JSContext* aJSContext, JSObject* aScopeArg,
-                        nsISupports* aCOMObj, const nsIID& aIID,
-                        JSObject** aRetVal) {
+nsresult nsIXPConnect::WrapNative(JSContext* aJSContext, JSObject* aScopeArg,
+                                  nsISupports* aCOMObj, const nsIID& aIID,
+                                  JSObject** aRetVal) {
   MOZ_ASSERT(aJSContext, "bad param");
   MOZ_ASSERT(aScopeArg, "bad param");
   MOZ_ASSERT(aCOMObj, "bad param");
@@ -605,11 +692,12 @@ nsXPConnect::WrapNative(JSContext* aJSContext, JSObject* aScopeArg,
   return NS_OK;
 }
 
-NS_IMETHODIMP
-nsXPConnect::WrapNativeToJSVal(JSContext* aJSContext, JSObject* aScopeArg,
-                               nsISupports* aCOMObj, nsWrapperCache* aCache,
-                               const nsIID* aIID, bool aAllowWrapping,
-                               MutableHandleValue aVal) {
+nsresult nsIXPConnect::WrapNativeToJSVal(JSContext* aJSContext,
+                                         JSObject* aScopeArg,
+                                         nsISupports* aCOMObj,
+                                         nsWrapperCache* aCache,
+                                         const nsIID* aIID, bool aAllowWrapping,
+                                         MutableHandleValue aVal) {
   MOZ_ASSERT(aJSContext, "bad param");
   MOZ_ASSERT(aScopeArg, "bad param");
   MOZ_ASSERT(aCOMObj, "bad param");
@@ -619,9 +707,8 @@ nsXPConnect::WrapNativeToJSVal(JSContext* aJSContext, JSObject* aScopeArg,
                                   aAllowWrapping, aVal);
 }
 
-NS_IMETHODIMP
-nsXPConnect::WrapJS(JSContext* aJSContext, JSObject* aJSObjArg,
-                    const nsIID& aIID, void** result) {
+nsresult nsIXPConnect::WrapJS(JSContext* aJSContext, JSObject* aJSObjArg,
+                              const nsIID& aIID, void** result) {
   MOZ_ASSERT(aJSContext, "bad param");
   MOZ_ASSERT(aJSObjArg, "bad param");
   MOZ_ASSERT(result, "bad param");
@@ -637,9 +724,8 @@ nsXPConnect::WrapJS(JSContext* aJSContext, JSObject* aJSObjArg,
   return NS_OK;
 }
 
-NS_IMETHODIMP
-nsXPConnect::JSValToVariant(JSContext* cx, HandleValue aJSVal,
-                            nsIVariant** aResult) {
+nsresult nsIXPConnect::JSValToVariant(JSContext* cx, HandleValue aJSVal,
+                                      nsIVariant** aResult) {
   MOZ_ASSERT(aResult, "bad param");
 
   RefPtr<XPCVariant> variant = XPCVariant::newVariant(cx, aJSVal);
@@ -649,11 +735,11 @@ nsXPConnect::JSValToVariant(JSContext* cx, HandleValue aJSVal,
   return NS_OK;
 }
 
-NS_IMETHODIMP
-nsXPConnect::WrapJSAggregatedToNative(nsISupports* aOuter,
-                                      JSContext* aJSContext,
-                                      JSObject* aJSObjArg, const nsIID& aIID,
-                                      void** result) {
+nsresult nsIXPConnect::WrapJSAggregatedToNative(nsISupports* aOuter,
+                                                JSContext* aJSContext,
+                                                JSObject* aJSObjArg,
+                                                const nsIID& aIID,
+                                                void** result) {
   MOZ_ASSERT(aOuter, "bad param");
   MOZ_ASSERT(aJSContext, "bad param");
   MOZ_ASSERT(aJSObjArg, "bad param");
@@ -669,10 +755,9 @@ nsXPConnect::WrapJSAggregatedToNative(nsISupports* aOuter,
   return NS_OK;
 }
 
-NS_IMETHODIMP
-nsXPConnect::GetWrappedNativeOfJSObject(JSContext* aJSContext,
-                                        JSObject* aJSObjArg,
-                                        nsIXPConnectWrappedNative** _retval) {
+nsresult nsIXPConnect::GetWrappedNativeOfJSObject(
+    JSContext* aJSContext, JSObject* aJSObjArg,
+    nsIXPConnectWrappedNative** _retval) {
   MOZ_ASSERT(aJSContext, "bad param");
   MOZ_ASSERT(aJSObjArg, "bad param");
   MOZ_ASSERT(_retval, "bad param");
@@ -680,7 +765,7 @@ nsXPConnect::GetWrappedNativeOfJSObject(JSContext* aJSContext,
   RootedObject aJSObj(aJSContext, aJSObjArg);
   aJSObj = js::CheckedUnwrapDynamic(aJSObj, aJSContext,
                                     /* stopAtWindowProxy = */ false);
-  if (!aJSObj || !IS_WN_REFLECTOR(aJSObj)) {
+  if (!aJSObj || !IsWrappedNativeReflector(aJSObj)) {
     *_retval = nullptr;
     return NS_ERROR_FAILURE;
   }
@@ -696,7 +781,7 @@ static already_AddRefed<nsISupports> ReflectorToISupports(JSObject* reflector) {
   }
 
   // Try XPCWrappedNatives.
-  if (IS_WN_REFLECTOR(reflector)) {
+  if (IsWrappedNativeReflector(reflector)) {
     XPCWrappedNative* wn = XPCWrappedNative::Get(reflector);
     if (!wn) {
       return nullptr;
@@ -727,9 +812,8 @@ already_AddRefed<nsISupports> xpc::ReflectorToISupportsDynamic(
                                /* stopAtWindowProxy = */ false));
 }
 
-NS_IMETHODIMP
-nsXPConnect::CreateSandbox(JSContext* cx, nsIPrincipal* principal,
-                           JSObject** _retval) {
+nsresult nsIXPConnect::CreateSandbox(JSContext* cx, nsIPrincipal* principal,
+                                     JSObject** _retval) {
   *_retval = nullptr;
 
   RootedValue rval(cx);
@@ -745,10 +829,10 @@ nsXPConnect::CreateSandbox(JSContext* cx, nsIPrincipal* principal,
   return rv;
 }
 
-NS_IMETHODIMP
-nsXPConnect::EvalInSandboxObject(const nsAString& source, const char* filename,
-                                 JSContext* cx, JSObject* sandboxArg,
-                                 MutableHandleValue rval) {
+nsresult nsIXPConnect::EvalInSandboxObject(const nsAString& source,
+                                           const char* filename, JSContext* cx,
+                                           JSObject* sandboxArg,
+                                           MutableHandleValue rval) {
   if (!sandboxArg) {
     return NS_ERROR_INVALID_ARG;
   }
@@ -764,28 +848,28 @@ nsXPConnect::EvalInSandboxObject(const nsAString& source, const char* filename,
                        /* enforceFilenameRestrictions */ true, rval);
 }
 
-NS_IMETHODIMP
-nsXPConnect::DebugDump(int16_t depth) {
+nsresult nsIXPConnect::DebugDump(int16_t depth) {
 #ifdef DEBUG
+  auto* self = static_cast<nsXPConnect*>(this);
+
   depth--;
   XPC_LOG_ALWAYS(
-      ("nsXPConnect @ %p with mRefCnt = %" PRIuPTR, this, mRefCnt.get()));
+      ("nsXPConnect @ %p with mRefCnt = %" PRIuPTR, self, self->mRefCnt.get()));
   XPC_LOG_INDENT();
-  XPC_LOG_ALWAYS(("gSelf @ %p", gSelf));
-  XPC_LOG_ALWAYS(("gOnceAliveNowDead is %d", (int)gOnceAliveNowDead));
+  XPC_LOG_ALWAYS(("gSelf @ %p", self->gSelf));
+  XPC_LOG_ALWAYS(("gOnceAliveNowDead is %d", (int)self->gOnceAliveNowDead));
   XPCWrappedNativeScope::DebugDumpAllScopes(depth);
   XPC_LOG_OUTDENT();
 #endif
   return NS_OK;
 }
 
-NS_IMETHODIMP
-nsXPConnect::DebugDumpObject(nsISupports* p, int16_t depth) {
+nsresult nsIXPConnect::DebugDumpObject(nsISupports* aCOMObj, int16_t depth) {
 #ifdef DEBUG
   if (!depth) {
     return NS_OK;
   }
-  if (!p) {
+  if (!aCOMObj) {
     XPC_LOG_ALWAYS(("*** Cound not dump object with NULL address"));
     return NS_OK;
   }
@@ -794,36 +878,35 @@ nsXPConnect::DebugDumpObject(nsISupports* p, int16_t depth) {
   nsCOMPtr<nsIXPConnectWrappedNative> wn;
   nsCOMPtr<nsIXPConnectWrappedJS> wjs;
 
-  if (NS_SUCCEEDED(
-          p->QueryInterface(NS_GET_IID(nsIXPConnect), getter_AddRefs(xpc)))) {
+  if (NS_SUCCEEDED(aCOMObj->QueryInterface(NS_GET_IID(nsIXPConnect),
+                                           getter_AddRefs(xpc)))) {
     XPC_LOG_ALWAYS(("Dumping a nsIXPConnect..."));
     xpc->DebugDump(depth);
-  } else if (NS_SUCCEEDED(p->QueryInterface(
+  } else if (NS_SUCCEEDED(aCOMObj->QueryInterface(
                  NS_GET_IID(nsIXPConnectWrappedNative), getter_AddRefs(wn)))) {
     XPC_LOG_ALWAYS(("Dumping a nsIXPConnectWrappedNative..."));
     wn->DebugDump(depth);
-  } else if (NS_SUCCEEDED(p->QueryInterface(NS_GET_IID(nsIXPConnectWrappedJS),
-                                            getter_AddRefs(wjs)))) {
+  } else if (NS_SUCCEEDED(aCOMObj->QueryInterface(
+                 NS_GET_IID(nsIXPConnectWrappedJS), getter_AddRefs(wjs)))) {
     XPC_LOG_ALWAYS(("Dumping a nsIXPConnectWrappedJS..."));
     wjs->DebugDump(depth);
   } else {
-    XPC_LOG_ALWAYS(("*** Could not dump the nsISupports @ %p", p));
+    XPC_LOG_ALWAYS(("*** Could not dump the nsISupports @ %p", aCOMObj));
   }
 #endif
   return NS_OK;
 }
 
-NS_IMETHODIMP
-nsXPConnect::DebugDumpJSStack(bool showArgs, bool showLocals,
-                              bool showThisProps) {
+nsresult nsIXPConnect::DebugDumpJSStack(bool showArgs, bool showLocals,
+                                        bool showThisProps) {
   xpc_DumpJSStack(showArgs, showLocals, showThisProps);
 
   return NS_OK;
 }
 
-NS_IMETHODIMP
-nsXPConnect::VariantToJS(JSContext* ctx, JSObject* scopeArg, nsIVariant* value,
-                         MutableHandleValue _retval) {
+nsresult nsIXPConnect::VariantToJS(JSContext* ctx, JSObject* scopeArg,
+                                   nsIVariant* value,
+                                   MutableHandleValue _retval) {
   MOZ_ASSERT(ctx, "bad param");
   MOZ_ASSERT(scopeArg, "bad param");
   MOZ_ASSERT(value, "bad param");
@@ -842,9 +925,8 @@ nsXPConnect::VariantToJS(JSContext* ctx, JSObject* scopeArg, nsIVariant* value,
   return NS_OK;
 }
 
-NS_IMETHODIMP
-nsXPConnect::JSToVariant(JSContext* ctx, HandleValue value,
-                         nsIVariant** _retval) {
+nsresult nsIXPConnect::JSToVariant(JSContext* ctx, HandleValue value,
+                                   nsIVariant** _retval) {
   MOZ_ASSERT(ctx, "bad param");
   MOZ_ASSERT(_retval, "bad param");
 
@@ -919,110 +1001,6 @@ void SetLocationForGlobal(JSObject* global, nsIURI* locationURI) {
 
 }  // namespace xpc
 
-NS_IMETHODIMP
-nsXPConnect::WriteScript(nsIObjectOutputStream* stream, JSContext* cx,
-                         JSScript* scriptArg) {
-  RootedScript script(cx, scriptArg);
-
-  uint8_t flags = 0;  // We don't have flags anymore.
-  nsresult rv = stream->Write8(flags);
-  if (NS_FAILED(rv)) {
-    return rv;
-  }
-
-  TranscodeBuffer buffer;
-  TranscodeResult code;
-  code = EncodeScript(cx, buffer, script);
-
-  if (code != TranscodeResult::Ok) {
-    if (code == TranscodeResult::Throw) {
-      JS_ClearPendingException(cx);
-      return NS_ERROR_OUT_OF_MEMORY;
-    }
-
-    MOZ_ASSERT(IsTranscodeFailureResult(code));
-    return NS_ERROR_FAILURE;
-  }
-
-  size_t size = buffer.length();
-  if (size > UINT32_MAX) {
-    return NS_ERROR_FAILURE;
-  }
-  rv = stream->Write32(size);
-  if (NS_SUCCEEDED(rv)) {
-    // Ideally we could just pass "buffer" here.  See bug 1566574.
-    rv = stream->WriteBytes(Span(buffer.begin(), size));
-  }
-
-  return rv;
-}
-
-NS_IMETHODIMP
-nsXPConnect::ReadScript(nsIObjectInputStream* stream, JSContext* cx,
-                        const JS::ReadOnlyCompileOptions& options,
-                        JSScript** scriptp) {
-  uint8_t flags;
-  nsresult rv = stream->Read8(&flags);
-  if (NS_FAILED(rv)) {
-    return rv;
-  }
-
-  // We don't serialize mutedError-ness of scripts, which is fine as long as
-  // we only serialize system and XUL-y things. We can detect this by checking
-  // where the caller wants us to deserialize.
-  //
-  // CompilationScope() could theoretically GC, so get that out of the way
-  // before comparing to the cx global.
-  JSObject* loaderGlobal = xpc::CompilationScope();
-  MOZ_RELEASE_ASSERT(nsContentUtils::IsSystemCaller(cx) ||
-                     CurrentGlobalOrNull(cx) == loaderGlobal);
-
-  uint32_t size;
-  rv = stream->Read32(&size);
-  if (NS_FAILED(rv)) {
-    return rv;
-  }
-
-  char* data;
-  rv = stream->ReadBytes(size, &data);
-  if (NS_FAILED(rv)) {
-    return rv;
-  }
-
-  TranscodeBuffer buffer;
-  buffer.replaceRawBuffer(reinterpret_cast<uint8_t*>(data), size);
-
-  {
-    TranscodeResult code;
-    Rooted<JSScript*> script(cx);
-    code = DecodeScript(cx, options, buffer, &script);
-    if (code == TranscodeResult::Ok) {
-      *scriptp = script.get();
-    } else {
-      if (code == TranscodeResult::Throw) {
-        JS_ClearPendingException(cx);
-        return NS_ERROR_OUT_OF_MEMORY;
-      }
-
-      MOZ_ASSERT(IsTranscodeFailureResult(code));
-      return NS_ERROR_FAILURE;
-    }
-  }
-
-  return rv;
-}
-
-NS_IMETHODIMP
-nsXPConnect::GetIsShuttingDown(bool* aIsShuttingDown) {
-  if (!aIsShuttingDown) {
-    return NS_ERROR_INVALID_ARG;
-  }
-
-  *aIsShuttingDown = mShuttingDown;
-
-  return NS_OK;
-}
-
 // static
 nsIXPConnect* nsIXPConnect::XPConnect() {
   // Do a release-mode assert that we're not doing anything significant in
@@ -1052,7 +1030,7 @@ MOZ_EXPORT void DumpCompleteHeap() {
     return;
   }
 
-  nsJSContext::CycleCollectNow(alltracesListener);
+  nsJSContext::CycleCollectNow(CCReason::DUMP_HEAP, alltracesListener);
 }
 
 }  // extern "C"
