@@ -4,18 +4,15 @@
 
 #include "xpctest_private.h"
 #include "nsComponentManagerUtils.h"
+#include "nsImportModule.h"
 
 NS_IMPL_ISUPPORTS(nsXPCTestReturnCodeParent, nsIXPCTestReturnCodeParent)
 
-nsXPCTestReturnCodeParent::nsXPCTestReturnCodeParent() = default;
-
-nsXPCTestReturnCodeParent::~nsXPCTestReturnCodeParent() = default;
-
 NS_IMETHODIMP nsXPCTestReturnCodeParent::CallChild(int32_t childBehavior,
                                                    nsresult* _retval) {
   nsresult rv;
-  nsCOMPtr<nsIXPCTestReturnCodeChild> child(
-      do_CreateInstance("@mozilla.org/js/xpc/test/js/ReturnCodeChild;1", &rv));
+  nsCOMPtr<nsIXPCTestReturnCodeChild> child(do_ImportModule(
+      "resource://test/ReturnCodeChild.jsm", "ReturnCodeChild", &rv));
   NS_ENSURE_SUCCESS(rv, rv);
   rv = child->DoIt(childBehavior);
   *_retval = rv;
