@@ -16,22 +16,16 @@ uint64_t GfxDriverInfo::allDriverVersions = ~(uint64_t(0));
 
 GfxDeviceFamily*
     GfxDriverInfo::sDeviceFamilies[static_cast<size_t>(DeviceFamily::Max)];
-nsAString* GfxDriverInfo::sDesktopEnvironment[static_cast<size_t>(
-    DesktopEnvironment::Max)];
-nsAString*
+nsString*
     GfxDriverInfo::sWindowProtocol[static_cast<size_t>(WindowProtocol::Max)];
-nsAString*
-    GfxDriverInfo::sDeviceVendors[static_cast<size_t>(DeviceVendor::Max)];
-nsAString*
-    GfxDriverInfo::sDriverVendors[static_cast<size_t>(DriverVendor::Max)];
+nsString* GfxDriverInfo::sDeviceVendors[static_cast<size_t>(DeviceVendor::Max)];
+nsString* GfxDriverInfo::sDriverVendors[static_cast<size_t>(DriverVendor::Max)];
 
 GfxDriverInfo::GfxDriverInfo()
     : mOperatingSystem(OperatingSystem::Unknown),
       mOperatingSystemVersion(0),
       mScreen(ScreenSizeStatus::All),
       mBattery(BatteryStatus::All),
-      mDesktopEnvironment(
-          GfxDriverInfo::GetDesktopEnvironment(DesktopEnvironment::All)),
       mWindowProtocol(GfxDriverInfo::GetWindowProtocol(WindowProtocol::All)),
       mAdapterVendor(GfxDriverInfo::GetDeviceVendor(DeviceFamily::All)),
       mDriverVendor(GfxDriverInfo::GetDriverVendor(DriverVendor::All)),
@@ -48,17 +42,15 @@ GfxDriverInfo::GfxDriverInfo()
 
 GfxDriverInfo::GfxDriverInfo(
     OperatingSystem os, ScreenSizeStatus screen, BatteryStatus battery,
-    const nsAString& desktopEnv, const nsAString& windowProtocol,
-    const nsAString& vendor, const nsAString& driverVendor,
-    GfxDeviceFamily* devices, int32_t feature, int32_t featureStatus,
-    VersionComparisonOp op, uint64_t driverVersion, const char* ruleId,
-    const char* suggestedVersion /* = nullptr */, bool ownDevices /* = false */,
-    bool gpu2 /* = false */)
+    const nsAString& windowProtocol, const nsAString& vendor,
+    const nsAString& driverVendor, GfxDeviceFamily* devices, int32_t feature,
+    int32_t featureStatus, VersionComparisonOp op, uint64_t driverVersion,
+    const char* ruleId, const char* suggestedVersion /* = nullptr */,
+    bool ownDevices /* = false */, bool gpu2 /* = false */)
     : mOperatingSystem(os),
       mOperatingSystemVersion(0),
       mScreen(screen),
       mBattery(battery),
-      mDesktopEnvironment(desktopEnv),
       mWindowProtocol(windowProtocol),
       mAdapterVendor(vendor),
       mDriverVendor(driverVendor),
@@ -78,7 +70,6 @@ GfxDriverInfo::GfxDriverInfo(const GfxDriverInfo& aOrig)
       mOperatingSystemVersion(aOrig.mOperatingSystemVersion),
       mScreen(aOrig.mScreen),
       mBattery(aOrig.mBattery),
-      mDesktopEnvironment(aOrig.mDesktopEnvironment),
       mWindowProtocol(aOrig.mWindowProtocol),
       mAdapterVendor(aOrig.mAdapterVendor),
       mDriverVendor(aOrig.mDriverVendor),
@@ -394,7 +385,7 @@ const GfxDeviceFamily* GfxDriverInfo::GetDeviceFamily(DeviceFamily id) {
       APPEND_DEVICE(0x6779);
       APPEND_DEVICE(0x677b);
       break;
-    case DeviceFamily::RadeonBlockNoVideoCopy:
+    case DeviceFamily::RadeonBlockZeroVideoCopy:
       // Stoney
       APPEND_DEVICE(0x98e4);
       // Carrizo
@@ -509,12 +500,6 @@ const GfxDeviceFamily* GfxDriverInfo::GetDeviceFamily(DeviceFamily id) {
       APPEND_RANGE(0x0190, 0x019e);  // early tesla
       APPEND_RANGE(0x0500, 0x05df);  // C67-C68
       break;
-    case DeviceFamily::NvidiaRolloutWebRender:
-      APPEND_RANGE(0x0400, 0x04ff);
-      APPEND_RANGE(0x05e0, 0x05ff);
-      APPEND_RANGE(0x0600, INT32_MAX);
-      APPEND_RANGE(0x06c0, INT32_MAX);
-      break;
     case DeviceFamily::IntelWebRenderBlocked:
       // powervr
       // sgx535
@@ -563,298 +548,6 @@ const GfxDeviceFamily* GfxDriverInfo::GetDeviceFamily(DeviceFamily id) {
       // gen5 (ironlake)
       APPEND_DEVICE(0x0042);
       APPEND_DEVICE(0x0046);
-      break;
-    case DeviceFamily::IntelRolloutWebRender:
-      // Disable WebRender on these devices for now
-      // to match what's going out into release
-#if 0
-      // gen4.5 - G45
-      APPEND_DEVICE(0x2e22);
-
-      // gen5 (ironlake)
-      APPEND_DEVICE(0x0042);
-      APPEND_DEVICE(0x0046);
-#endif
-
-      // cherryview
-      APPEND_DEVICE(0x22b0);
-      APPEND_DEVICE(0x22b1);
-      APPEND_DEVICE(0x22b2);
-      APPEND_DEVICE(0x22b3);
-
-      [[fallthrough]];
-    case DeviceFamily::IntelModernRolloutWebRender:
-      // Temporarily disable WebRender on gen6 to get more sw-wr+D3D11 testing
-#ifndef NIGHTLY_BUILD
-      // sandybridge gen6 gt1
-      APPEND_DEVICE(0x0102);
-      APPEND_DEVICE(0x0106);
-      APPEND_DEVICE(0x010a);
-
-      // sandybridge gen6 gt2
-      APPEND_DEVICE(0x0112);
-      APPEND_DEVICE(0x0116);
-      APPEND_DEVICE(0x0122);
-      APPEND_DEVICE(0x0126);
-#endif
-
-      // ivybridge gen7 baytrail
-      APPEND_DEVICE(0x0f30);
-      APPEND_DEVICE(0x0f31);
-      APPEND_DEVICE(0x0f33);
-      APPEND_DEVICE(0x0155);
-      APPEND_DEVICE(0x0157);
-
-      // ivybridge gen7 gt1
-      APPEND_DEVICE(0x0152);
-      APPEND_DEVICE(0x0156);
-      APPEND_DEVICE(0x015a);
-
-      // ivybridge gen7 gt2
-      APPEND_DEVICE(0x0162);
-      APPEND_DEVICE(0x0166);
-      APPEND_DEVICE(0x016a);
-
-      // haswell gen7.5 gt1
-      APPEND_DEVICE(0x0402);
-      APPEND_DEVICE(0x0406);
-      APPEND_DEVICE(0x040a);
-      APPEND_DEVICE(0x040b);
-      APPEND_DEVICE(0x040e);
-      APPEND_DEVICE(0x0a02);
-      APPEND_DEVICE(0x0a06);
-      APPEND_DEVICE(0x0a0a);
-      APPEND_DEVICE(0x0a0b);
-      APPEND_DEVICE(0x0a0e);
-      APPEND_DEVICE(0x0c02);
-      APPEND_DEVICE(0x0c06);
-      APPEND_DEVICE(0x0c0c);
-      APPEND_DEVICE(0x0c0b);
-      APPEND_DEVICE(0x0c0e);
-      APPEND_DEVICE(0x0d02);
-      APPEND_DEVICE(0x0d06);
-      APPEND_DEVICE(0x0d0a);
-      APPEND_DEVICE(0x0d0b);
-      APPEND_DEVICE(0x0d0e);
-
-      // haswell gen7.5 gt2
-      APPEND_DEVICE(0x0412);
-      APPEND_DEVICE(0x0416);
-      APPEND_DEVICE(0x041a);
-      APPEND_DEVICE(0x041b);
-      APPEND_DEVICE(0x041e);
-      APPEND_DEVICE(0x0a12);
-      APPEND_DEVICE(0x0a16);
-      APPEND_DEVICE(0x0a1a);
-      APPEND_DEVICE(0x0a1b);
-      APPEND_DEVICE(0x0a1e);
-
-      // haswell gen7.5 gt3
-      APPEND_DEVICE(0x0422);
-      APPEND_DEVICE(0x0426);
-      APPEND_DEVICE(0x042a);
-      APPEND_DEVICE(0x042b);
-      APPEND_DEVICE(0x042e);
-      APPEND_DEVICE(0x0a22);
-      APPEND_DEVICE(0x0a26);
-      APPEND_DEVICE(0x0a0a);
-      APPEND_DEVICE(0x0a1a);
-      APPEND_DEVICE(0x0a2a);
-      APPEND_DEVICE(0x0a2b);
-      APPEND_DEVICE(0x0a2e);
-      APPEND_DEVICE(0x0c22);
-      APPEND_DEVICE(0x0c26);
-      APPEND_DEVICE(0x0c2c);
-      APPEND_DEVICE(0x0c2b);
-      APPEND_DEVICE(0x0c2e);
-      APPEND_DEVICE(0x0d22);
-      APPEND_DEVICE(0x0d26);
-      APPEND_DEVICE(0x0d2b);
-      APPEND_DEVICE(0x0d2e);
-
-      // broxton (apollolake)
-      APPEND_DEVICE(0x0a84);
-      APPEND_DEVICE(0x1a84);
-      APPEND_DEVICE(0x1a85);
-      APPEND_DEVICE(0x5a84);
-      APPEND_DEVICE(0x5a85);
-
-      // geminilake
-      APPEND_DEVICE(0x3184);
-      APPEND_DEVICE(0x3185);
-
-      // broadwell gt1 (gen8)
-      APPEND_DEVICE(0x1602);
-      APPEND_DEVICE(0x1606);
-      APPEND_DEVICE(0x160a);
-      APPEND_DEVICE(0x160b);
-      APPEND_DEVICE(0x160d);
-      APPEND_DEVICE(0x160e);
-
-      // broadwell gt2+ (gen8)
-      APPEND_DEVICE(0x1612);
-      APPEND_DEVICE(0x1616);
-      APPEND_DEVICE(0x161a);
-      APPEND_DEVICE(0x161b);
-      APPEND_DEVICE(0x161d);
-      APPEND_DEVICE(0x161e);
-      APPEND_DEVICE(0x1622);
-      APPEND_DEVICE(0x1626);
-      APPEND_DEVICE(0x162a);
-      APPEND_DEVICE(0x162b);
-      APPEND_DEVICE(0x162d);
-      APPEND_DEVICE(0x162e);
-      APPEND_DEVICE(0x1632);
-      APPEND_DEVICE(0x1636);
-      APPEND_DEVICE(0x163a);
-      APPEND_DEVICE(0x163b);
-      APPEND_DEVICE(0x163d);
-      APPEND_DEVICE(0x163e);
-
-      // skylake gt1
-      APPEND_DEVICE(0x1902);
-      APPEND_DEVICE(0x1906);
-      APPEND_DEVICE(0x190a);
-      APPEND_DEVICE(0x190e);
-
-      // skylake gt2
-      APPEND_DEVICE(0x1912);
-      APPEND_DEVICE(0x1913);
-      APPEND_DEVICE(0x1915);
-      APPEND_DEVICE(0x1916);
-      APPEND_DEVICE(0x1917);
-      APPEND_DEVICE(0x191a);
-      APPEND_DEVICE(0x191b);
-      APPEND_DEVICE(0x191d);
-      APPEND_DEVICE(0x191e);
-      APPEND_DEVICE(0x1921);
-
-      // skylake gt3
-      APPEND_DEVICE(0x1923);
-      APPEND_DEVICE(0x1926);
-      APPEND_DEVICE(0x1927);
-      APPEND_DEVICE(0x192b);
-
-      // skylake gt4
-      APPEND_DEVICE(0x1932);
-      APPEND_DEVICE(0x193a);
-      APPEND_DEVICE(0x193b);
-      APPEND_DEVICE(0x193d);
-
-      // kabylake gt1
-      APPEND_DEVICE(0x5902);
-      APPEND_DEVICE(0x5906);
-      APPEND_DEVICE(0x5908);
-      APPEND_DEVICE(0x590a);
-      APPEND_DEVICE(0x590b);
-      APPEND_DEVICE(0x590e);
-
-      // kabylake gt1.5
-      APPEND_DEVICE(0x5913);
-      APPEND_DEVICE(0x5915);
-      APPEND_DEVICE(0x5917);
-
-      // kabylake gt2
-      APPEND_DEVICE(0x5912);
-      APPEND_DEVICE(0x5916);
-      APPEND_DEVICE(0x591a);
-      APPEND_DEVICE(0x591b);
-      APPEND_DEVICE(0x591c);
-      APPEND_DEVICE(0x591d);
-      APPEND_DEVICE(0x591e);
-      APPEND_DEVICE(0x5921);
-      APPEND_DEVICE(0x5926);
-      APPEND_DEVICE(0x5923);
-      APPEND_DEVICE(0x5927);
-      APPEND_DEVICE(0x593b);
-      APPEND_DEVICE(0x87c0);
-
-      // coffeelake gt1
-      APPEND_DEVICE(0x3e90);
-      APPEND_DEVICE(0x3e93);
-      APPEND_DEVICE(0x3e99);
-      APPEND_DEVICE(0x3e9c);
-      APPEND_DEVICE(0x3ea1);
-      APPEND_DEVICE(0x3ea4);
-      APPEND_DEVICE(0x9b21);
-      APPEND_DEVICE(0x9ba0);
-      APPEND_DEVICE(0x9ba2);
-      APPEND_DEVICE(0x9ba4);
-      APPEND_DEVICE(0x9ba5);
-      APPEND_DEVICE(0x9ba8);
-      APPEND_DEVICE(0x9baa);
-      APPEND_DEVICE(0x9bab);
-      APPEND_DEVICE(0x9bac);
-
-      // coffeelake gt2+
-      APPEND_RANGE(0x3e91, 0x3e92);
-      APPEND_DEVICE(0x3e94);
-      APPEND_DEVICE(0x3e96);
-      APPEND_DEVICE(0x3e98);
-      APPEND_RANGE(0x3e9a, 0x3e9b);
-      APPEND_DEVICE(0x3ea0);
-      APPEND_DEVICE(0x3ea2);
-      APPEND_RANGE(0x3ea5, 0x3ea9);
-      APPEND_DEVICE(0x87ca);
-      APPEND_DEVICE(0x9b41);
-      APPEND_DEVICE(0x9bc0);
-      APPEND_DEVICE(0x9bc2);
-      APPEND_RANGE(0x9bc4, 0x9bf6);
-
-      // icelake gt1,gt1.5,gt2
-      APPEND_RANGE(0x8a50, 0x8a5d);
-
-      // rocketlake
-      APPEND_RANGE(0x4c8a, 0x4c9a);
-
-      // tigerlake
-      APPEND_RANGE(0x9a40, 0x9af8);
-      break;
-    case DeviceFamily::AtiRolloutWebRender:
-      APPEND_RANGE(0x6600, 0x66af);
-      APPEND_RANGE(0x6700, 0x671f);
-      APPEND_RANGE(0x6780, 0x683f);
-      APPEND_RANGE(0x6860, 0x687f);
-      APPEND_RANGE(0x6900, 0x69ff);
-      APPEND_DEVICE(0x6fdf);
-      APPEND_DEVICE(0x7300);
-      APPEND_RANGE(0x7310, 0x73ff);
-      APPEND_RANGE(0x9830, 0x986f);
-      APPEND_RANGE(0x9900, 0x99ff);
-      // Raven
-      APPEND_DEVICE(0x15dd);
-      APPEND_DEVICE(0x15d8);
-      // Renoir
-      APPEND_DEVICE(0x1636);
-      // Cezanne
-      APPEND_DEVICE(0x1638);
-      // Lucienne
-      APPEND_DEVICE(0x164c);
-      // Yellow Carp
-      APPEND_DEVICE(0x1681);
-
-      // Evergreen
-      APPEND_RANGE(0x6840, 0x684b);
-      APPEND_RANGE(0x6850, 0x685f);
-      APPEND_RANGE(0x6880, 0x68ff);
-      APPEND_RANGE(0x9800, 0x980a);
-      APPEND_RANGE(0x9640, 0x964f);
-      APPEND_RANGE(0x6720, 0x677f);
-
-      // Stoney
-      APPEND_DEVICE(0x98e4);
-
-      // Carrizo
-      APPEND_RANGE(0x9870, 0x9877);
-
-      // Kaveri
-      APPEND_RANGE(0x1304, 0x131d);
-
-      // R700
-      APPEND_RANGE(0x9440, 0x949f);
-      APPEND_RANGE(0x94a0, 0x94b9);
-      APPEND_RANGE(0x9540, 0x955f);
-
       break;
     // This should never happen, but we get a warning if we don't handle this.
     case DeviceFamily::Max:
@@ -874,50 +567,6 @@ const GfxDeviceFamily* GfxDriverInfo::GetDeviceFamily(DeviceFamily id) {
   return deviceFamily;
 }
 
-// Macro for assigning a desktop environment to a string.
-#define DECLARE_DESKTOP_ENVIRONMENT_ID(name, desktopEnvId) \
-  case DesktopEnvironment::name:                           \
-    sDesktopEnvironment[idx]->AssignLiteral(desktopEnvId); \
-    break;
-
-const nsAString& GfxDriverInfo::GetDesktopEnvironment(DesktopEnvironment id) {
-  if (id >= DesktopEnvironment::Max) {
-    MOZ_ASSERT_UNREACHABLE("DesktopEnvironment id is out of range");
-    id = DesktopEnvironment::All;
-  }
-
-  auto idx = static_cast<size_t>(id);
-  if (sDesktopEnvironment[idx]) {
-    return *sDesktopEnvironment[idx];
-  }
-
-  sDesktopEnvironment[idx] = new nsString();
-
-  switch (id) {
-    DECLARE_DESKTOP_ENVIRONMENT_ID(GNOME, "gnome");
-    DECLARE_DESKTOP_ENVIRONMENT_ID(KDE, "kde");
-    DECLARE_DESKTOP_ENVIRONMENT_ID(XFCE, "xfce");
-    DECLARE_DESKTOP_ENVIRONMENT_ID(Cinnamon, "cinnamon");
-    DECLARE_DESKTOP_ENVIRONMENT_ID(Enlightenment, "enlightment");
-    DECLARE_DESKTOP_ENVIRONMENT_ID(LXDE, "lxde");
-    DECLARE_DESKTOP_ENVIRONMENT_ID(Openbox, "openbox");
-    DECLARE_DESKTOP_ENVIRONMENT_ID(i3, "i3");
-    DECLARE_DESKTOP_ENVIRONMENT_ID(Sway, "sway");
-    DECLARE_DESKTOP_ENVIRONMENT_ID(Mate, "mate");
-    DECLARE_DESKTOP_ENVIRONMENT_ID(Unity, "unity");
-    DECLARE_DESKTOP_ENVIRONMENT_ID(Pantheon, "pantheon");
-    DECLARE_DESKTOP_ENVIRONMENT_ID(LXQT, "lxqt");
-    DECLARE_DESKTOP_ENVIRONMENT_ID(Deepin, "deepin");
-    DECLARE_DESKTOP_ENVIRONMENT_ID(Dwm, "dwm");
-    DECLARE_DESKTOP_ENVIRONMENT_ID(Budgie, "budgie");
-    DECLARE_DESKTOP_ENVIRONMENT_ID(Unknown, "unknown");
-    case DesktopEnvironment::Max:  // Suppress a warning.
-      DECLARE_DESKTOP_ENVIRONMENT_ID(All, "");
-  }
-
-  return *sDesktopEnvironment[idx];
-}
-
 // Macro for assigning a window protocol id to a string.
 #define DECLARE_WINDOW_PROTOCOL_ID(name, windowProtocolId) \
   case WindowProtocol::name:                               \
@@ -980,8 +629,6 @@ const nsAString& GfxDriverInfo::GetDeviceVendor(DeviceFamily id) {
     case DeviceFamily::IntelSkylake:
     case DeviceFamily::IntelHD520:
     case DeviceFamily::IntelMobileHDGraphics:
-    case DeviceFamily::IntelRolloutWebRender:
-    case DeviceFamily::IntelModernRolloutWebRender:
     case DeviceFamily::IntelWebRenderBlocked:
     case DeviceFamily::Bug1116812:
     case DeviceFamily::Bug1155608:
@@ -990,7 +637,6 @@ const nsAString& GfxDriverInfo::GetDeviceVendor(DeviceFamily id) {
       break;
     case DeviceFamily::NvidiaAll:
     case DeviceFamily::NvidiaBlockD3D9Layers:
-    case DeviceFamily::NvidiaRolloutWebRender:
     case DeviceFamily::NvidiaWebRenderBlocked:
     case DeviceFamily::Geforce7300GT:
     case DeviceFamily::Nvidia310M:
@@ -999,12 +645,11 @@ const nsAString& GfxDriverInfo::GetDeviceVendor(DeviceFamily id) {
       vendor = DeviceVendor::NVIDIA;
       break;
     case DeviceFamily::AtiAll:
-    case DeviceFamily::RadeonBlockNoVideoCopy:
+    case DeviceFamily::RadeonBlockZeroVideoCopy:
     case DeviceFamily::RadeonCaicos:
     case DeviceFamily::RadeonX1000:
     case DeviceFamily::Bug1447141:
     case DeviceFamily::AmdR600:
-    case DeviceFamily::AtiRolloutWebRender:
       vendor = DeviceVendor::ATI;
       break;
     case DeviceFamily::MicrosoftAll:
@@ -1099,6 +744,9 @@ const nsAString& GfxDriverInfo::GetDriverVendor(DriverVendor id) {
     DECLARE_DRIVER_VENDOR_ID(NonMesaAll, "non-mesa/all");
     DECLARE_DRIVER_VENDOR_ID(HardwareMesaAll, "mesa/hw-all");
     DECLARE_DRIVER_VENDOR_ID(SoftwareMesaAll, "mesa/sw-all");
+    DECLARE_DRIVER_VENDOR_ID(MesaNonIntelNvidiaAtiAll,
+                             "mesa/non-intel-nvidia-ati-all");
+    DECLARE_DRIVER_VENDOR_ID(MesaVM, "mesa/vmwgfx");
     case DriverVendor::Max:  // Suppress a warning.
       DECLARE_DRIVER_VENDOR_ID(All, "");
   }
